# Function to evaluate the Jacobian and Hessian of the residuals

# Inputs:
#=
-theta_in: Solution parameter
-parameters: Model Parameters
-ncoefs: Number of coefficients per decision rule, fixed at 7
=#

# Outputs:
#=
-h: Jacobian
-hh: Hessian
=#

function eval_hh_33_revised(theta_in::Array{Float64,1},parameters::Dict{String}{Float64})

# global ncoefs r psi1 psi2 rho_r pi_ss

ncoefs = 7;

psi1::Float64   = parameters["psi1"];
psi2::Float64   = parameters["psi2"];
pi_ss::Float64  = parameters["pi_ss"];
rho_r::Float64  = parameters["rho_r"];
r::Float64      = parameters["r"];

theta_nozlb = theta_in[1:3*ncoefs];
theta_zlb = theta_in[3*ncoefs+1:end];

theta_pi_1  = theta_nozlb[1:ncoefs];
theta_yy_1 = theta_nozlb[ncoefs+1:2*ncoefs];
theta_cc_1 = theta_nozlb[2*ncoefs+1:3*ncoefs];

theta_pi_2_temp = theta_zlb[1];
theta_yy_2_temp = theta_zlb[2];
theta_cc_2_temp = theta_zlb[3];

#Impose Coefficient Restrictions
a0 = theta_pi_1[1]; # const
a1 = theta_pi_1[2]; # eps_r
a2 = theta_pi_1[3]; # g-shock
a3 = theta_pi_1[4]; # z-shock
a4 = theta_pi_1[5]; #d-shock
a5 = theta_pi_1[6]; # yhat(-1)
a6 = theta_pi_1[7]; # Rhat(-1)

b0 = theta_yy_1[1]; # const
b1 = theta_yy_1[2]; # eps-r
b2 = theta_yy_1[3]; # g-shock
b3 = theta_yy_1[4]; # z-shock
b4 = theta_yy_1[5]; # d-shock
b5 = theta_yy_1[6]; # yhat(-1)
b6 = theta_yy_1[7]; # Rhat(-1)

c0 = theta_cc_1[1]; # const
c1 = theta_cc_1[2]; # eps-r
c2 = theta_cc_1[3]; # g-shock
c3 = theta_cc_1[4]; # z-shock
c4 = theta_cc_1[5]; # d-shock
c5 = theta_cc_1[6]; # yhat(-1)
c6 = theta_cc_1[7]; # Rhat(-1)

# ZLB regime - free coefficients
a13 = theta_pi_2_temp[1];
b13 = theta_yy_2_temp[1];
c13 = theta_cc_2_temp[1];

rhot = rho_r/(1-rho_r);


# Constraints evaluated at the free coefficients ( These are the mappings
# of the coefficients)

h =[ (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)), (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) - (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2), (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2), (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2), (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2), (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2), a13, b0 - ((b6 - b13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13), b1 - ((b6 - b13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), b2 - ((b6 - b13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), b3 - ((b6 - b13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), b4 - ((b6 - b13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), b5 - ((b6 - b13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), b13, c0 - ((c6 - c13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13), c1 - ((c6 - c13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), c2 - ((c6 - c13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), c3 - ((c6 - c13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), c4 - ((c6 - c13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), c5 - ((c6 - c13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13), c13];

hh =[(rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 0  log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) - (b0*psi2)/(rhot + a6*psi1 + b6*psi2) - (a0*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 + (b0*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (psi1*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 0   (a0*psi2)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a0*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (psi2*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) 0 0 0 0 0 0 0  (a0*psi1)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2)/(rhot + a6*psi1 + b6*psi2) - log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) 0 0;

0 (rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) - (b1*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi1*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) - (a1*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 + (b1*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 0 -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 (a1*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) + (b1*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a1*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 0 0 0 0 (a1*psi1)/(rhot + a6*psi1 + b6*psi2) - 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0;

0 0 (rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0 0 (b2*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a2*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b2*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0 -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) 0 0 0  (a2*psi2)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a2*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 0 0 0 0 (a2*psi1)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0;

0 0 0 (rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0   (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (b3*psi2)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) - (a3*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 + (b3*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) 0 0 (psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a3*psi2)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a3*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 0 0 0 0 psi2/(rhot + a6*psi1 + b6*psi2) + (a3*psi1)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0;

0 0 0 0 (rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) 0  (b4*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a4*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b4*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) 0  (a4*psi2)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a4*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2  0 0 0 0 0 0 0 (a4*psi1)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0;

0 0 0 0 0 (rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2)   psi2/(rhot + a6*psi1 + b6*psi2) - (b5*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a5*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 + (b5*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 0 0 -(psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) (a5*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (b5*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a5*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 0 0 0 0 0 0 0 (a5*psi1)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) + (b5*psi2)/(rhot + a6*psi1 + b6*psi2) 0 0;

0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0;

((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0 0 0  ((b6 - b13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13)^2 - ((b6 - b13)*((b0*psi2)/(rhot + a6*psi1 + b6*psi2) - log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (a0*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b0*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (psi1*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2)))/(a6 - a13)  1 - (psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 0  ((b6 - b13)*((a0*psi2)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a0*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (psi2*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2)))/(a6 - a13) - (a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) 0 0 0 0 0 0 0 ((b6 - b13)*((a0*psi1)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2)/(rhot + a6*psi1 + b6*psi2) - log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13) - ((b6 - b13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13)^2 (a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) 0;

0  ((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0 0   ((b6 - b13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((b6 - b13)*((b1*psi2)/(rhot + a6*psi1 + b6*psi2) - 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (psi1*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) + (a1*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b1*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 1-(psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 ((b6 - b13)*((a1*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) + (b1*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a1*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) - (a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0 0 0 0 0 0 0 ((b6 - b13)*((a1*psi1)/(rhot + a6*psi1 + b6*psi2) - 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((b6 - b13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2  (a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0;

0 0  ((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0   ((b6 - b13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((b6 - b13)*((b2*psi2)/(rhot + a6*psi1 + b6*psi2) + (a2*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b2*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0  1-(psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 ((b6 - b13)*((a2*psi2)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a2*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) - (a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0 0 0 0 0 0 0 ((b6 - b13)*((a2*psi1)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((b6 - b13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2  (a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0;

0 0 0  ((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0  ((b6 - b13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((b6 - b13)*(psi2/(rhot + a6*psi1 + b6*psi2) + (b3*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a3*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b3*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0  1-(psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2) 0 0 ((b6 - b13)*((psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a3*psi2)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a3*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) - (a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0 0 0 0 0 0 0 ((b6 - b13)*(psi2/(rhot + a6*psi1 + b6*psi2) + (a3*psi1)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((b6 - b13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2  (a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0;

0 0 0 0  ((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0  ((b6 - b13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((b6 - b13)*((b4*psi2)/(rhot + a6*psi1 + b6*psi2) + (a4*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b4*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0 0  1-(psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2) 0 ((b6 - b13)*((a4*psi2)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a4*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) - (a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0 0 0 0 0 0 0 ((b6 - b13)*((a4*psi1)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((b6 - b13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2  (a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0;

0 0 0 0 0 ((b6 - b13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13)  ((b6 - b13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((b6 - b13)*((b5*psi2)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) + (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a5*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b5*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0 0 0 1-(psi2*(b6 - b13))/(rhot + a6*psi1 + b6*psi2)  -(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) - ((b6 - b13)*((psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a5*psi2)/(rhot + a6*psi1 + b6*psi2) - (b5*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a5*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 0 0 0 0 0 0 ((b6 - b13)*((a5*psi1)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) + (b5*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((b6 - b13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2  (a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) 0;

0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0;

((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0 0 0  ((c6 - c13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13)^2 - ((c6 - c13)*((b0*psi2)/(rhot + a6*psi1 + b6*psi2) - log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (a0*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b0*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (psi1*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2)))/(a6 - a13) -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0 0 ((c6 - c13)*((a0*psi2)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a0*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (psi2*log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2)))/(a6 - a13)  1 0 0 0 0 0 -(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) ((c6 - c13)*((a0*psi1)/(rhot + a6*psi1 + b6*psi2) + (b0*psi2)/(rhot + a6*psi1 + b6*psi2) - log(pi_ss*r)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13) - ((c6 - c13)*(a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2))))/(a6 - a13)^2 0  (a0 - (a0*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b0*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) - (log(pi_ss*r)*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)))/(a6 - a13);

0  ((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0 0   ((c6 - c13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((c6 - c13)*((b1*psi2)/(rhot + a6*psi1 + b6*psi2) - 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (psi1*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) + (a1*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b1*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) 0 0 0 0  ((c6 - c13)*((a1*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)^2) + (b1*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a1*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 1 0 0 0 0  -(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13)  ((c6 - c13)*((a1*psi1)/(rhot + a6*psi1 + b6*psi2) - 1/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((c6 - c13)*(a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 0   (a1 - (a1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (a6 - a13)/((rho_r - 1)*(rhot + a6*psi1 + b6*psi2)) + (b1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13);

0 0  ((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0 0   ((c6 - c13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((c6 - c13)*((b2*psi2)/(rhot + a6*psi1 + b6*psi2) + (a2*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b2*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) 0 0 0   ((c6 - c13)*((a2*psi2)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a2*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 0 1 0 0 0 -(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13)   ((c6 - c13)*((a2*psi1)/(rhot + a6*psi1 + b6*psi2) + (b2*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((c6 - c13)*(a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 0  (a2 - (a2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b2*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13);

0 0 0  ((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0 0  ((c6 - c13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((c6 - c13)*(psi2/(rhot + a6*psi1 + b6*psi2) + (b3*psi2)/(rhot + a6*psi1 + b6*psi2) - (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a3*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b3*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0 -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) 0 0  ((c6 - c13)*((psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a3*psi2)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a3*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 0 0 1 0 0 -(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) ((c6 - c13)*(psi2/(rhot + a6*psi1 + b6*psi2) + (a3*psi1)/(rhot + a6*psi1 + b6*psi2) + (b3*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((c6 - c13)*(a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 0  (a3 - (a3*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b3*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13);

0 0 0 0  ((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13) 0   ((c6 - c13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((c6 - c13)*((b4*psi2)/(rhot + a6*psi1 + b6*psi2) + (a4*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b4*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0 0 -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) 0   ((c6 - c13)*((a4*psi2)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a4*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 0 0 0 1 0 -(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13)   ((c6 - c13)*((a4*psi1)/(rhot + a6*psi1 + b6*psi2) + (b4*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((c6 - c13)*(a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 0  (a4 - (a4*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) + (b4*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13);

0 0 0 0 0  ((c6 - c13)*((rhot + a13*psi1 + b6*psi2)/(rhot + a6*psi1 + b6*psi2) - 1))/(a6 - a13)  ((c6 - c13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 - ((c6 - c13)*((b5*psi2)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) + (psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a5*psi1*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2 - (b5*psi1*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13) 0 0 0 0 0 -(psi2*(c6 - c13))/(rhot + a6*psi1 + b6*psi2) -((c6 - c13)*((psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 - (a5*psi2)/(rhot + a6*psi1 + b6*psi2) - (b5*psi2^2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)^2 + (a5*psi2*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2)^2))/(a6 - a13)  0 0 0 0 0 1 -(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13) ((c6 - c13)*((a5*psi1)/(rhot + a6*psi1 + b6*psi2) - psi2/(rhot + a6*psi1 + b6*psi2) + (b5*psi2)/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13) - ((c6 - c13)*(a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2)))/(a6 - a13)^2 0  (a5 - (a5*(rhot + a13*psi1 + b6*psi2))/(rhot + a6*psi1 + b6*psi2) - (psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2) + (b5*psi2*(a6 - a13))/(rhot + a6*psi1 + b6*psi2))/(a6 - a13);

0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1];

return  h,hh

end
