
using Atom
using Distributions
using Clustering
using Optim
#using LsqFit
using Distances
using StatsBase
using DataFrames
using CSV
#using JLD
using Random
using LinearAlgebra
#using ControlSystems # for discrete Lyapunov equation solver

clearconsole()

#--------------------------------------------------------------------------
# Include procedures for model solution
#--------------------------------------------------------------------------
readDirSol = "$(pwd())/FunctionsSolution/"
include(readDirSol * "cga_grid_h.jl")
include(readDirSol * "cga_grid_km.jl")
include(readDirSol * "cga_grid_kms.jl")
include(readDirSol * "eval_hh_33_revised.jl")
include(readDirSol * "get_coefs_plc.jl")
include(readDirSol * "get_GH_nodes.jl")
include(readDirSol * "get_order1_sgu.jl")
include(readDirSol * "ghpoints.jl")
include(readDirSol * "gx_hx.jl")
include(readDirSol * "my_levenberg_marquardt2.jl")
include(readDirSol * "options.jl")
include(readDirSol * "PLC_solve.jl")
include(readDirSol * "PLC_main.jl")
#include(readDirSol * "simulate_plc.jl")
include(readDirSol * "solve_sgu.jl")
#include(readDirSol * "system_piecewise_linear_jacob_33_revised.jl")
include(readDirSol * "tsga_grid.jl")
include(readDirSol * "Monomials_2_N4.jl")

#--------------------------------------------------------------------------
# Include procedures for model estimation
#--------------------------------------------------------------------------
readDir = "FunctionsEstimation/"
include(readDir * "fCreateParamDict.jl")
include(readDir * "fAddSteadyStateParamDict.jl")
include(readDir * "fCanonicalForm.jl")
include(readDir * "fStateSpaceForm.jl")
include(readDir * "fStateSpaceSimulationaug.jl")
include(readDir * "fBSPF.jl")
include(readDir * "fPriorDraws.jl")
include(readDir * "fPriorDensity.jl")
include(readDir * "run_MCMC_BSPF.jl")
include(readDir * "fSystematicResampling.jl")

mhrun     = 3;   # mhrun=1 for 22000 draws and mhrun=2,3 for 55000 draws
nM        = 1000; # number of particles
ME_scale  = 0.1;
nSimRWMH  = 55000; # either 22000 or 55000
nBurnRWMH = 5000;  # either 2000 or 5000
silent    = 1;
nOutput   = 100;  # print output every ncount steps
c         = 0.005;  # scaling of proposal covariance matrix
                  # mhrun=1,2 for c=0.1 and mhrun=3 for c=0.005
rnd_off   = 0;    # control random seed
nSistResFilter = 0; # = 1 systematic resampling in filter, multinomial otherwise
pZLB = 0

#--------------------------------------------------------------------------
# Choose prior
#--------------------------------------------------------------------------

nNamePriorSpec = "Prior1"

#--------------------------------------------------------------------------
# Load data set
# date obs ygrowth (%) infl (annual %) R (annual %)
# 100*logcy_raw 100*logcy 100*logcy4
#--------------------------------------------------------------------------
nDataSet = "Sim1" # US or Sim1

if nDataSet == "US"
   mData    = CSV.read("Data/ACS-RED-USDATA-G.csv")
   nStartDate = 1
   nEndDate   = 140
   mData = convert(Array, mData);
   R_quantile = quantile(mData[nStartDate:nEndDate,5]./400 .+ 1,0.90) # R data quantile

else # if nDataSet == "Sim1"
   mData    = CSV.read("Data/COPFexactZLB_DGP1_Data.csv")
   nStartDate = 401
   nEndDate   = 540
   mData = convert(Array, mData);
   R_quantile = 1.01

end

#--------------------------------------------------------------------------
# Set solution type and Grid
#--------------------------------------------------------------------------
nSolutionRCLinear = 1
nSmolyakGrid = 1 # = 1 use Smolyak Grid. Note: need to set in options.jl file options_int_out["ext"] = 1 and options_int_out["k"] = 0

options_int_use,options_float_use = @eval options()

if nSmolyakGrid == 1
    options_int_use["k"]   = 0 # Number of non-linear solve steps
    options_int_use["ext"] = 1 # 1 = load cluster from input

    include("FunctionsSolutionSmolyak/Smolyak_Elem_isotrop.jl")
    include("FunctionsSolutionSmolyak/Smolyak_Grid.jl")
    include("FunctionsSolutionSmolyak/fGenSmolyakGrid.jl")
    include("FunctionsSolutionSmolyak/fScaleSmolyakGrid2.jl")

    d          = 6
    mu         = 2
    sd         = 1.282
    wSmolyak   = 1

    vSmolyakGridSpec = [d, mu, sd, wSmolyak, R_quantile]
    fixed_grid = fGenSmolyakGrid(d,mu,sd)

else
    fixed_grid = zeros(1,1)
    vSmolyakGridSpec = ones(1)
end

if nSolutionRCLinear == 1
    include(readDirSol * "simulate_plc_lin.jl")
    include(readDirSol * "system_piecewise_linear_jacob_33_lin.jl")
else
    include(readDirSol * "simulate_plc_nlin.jl")
    include(readDirSol * "system_piecewise_linear_jacob_33_nlin.jl")
end


if nSmolyakGrid == 1
    nGrid = "_GridSmolyak"
else
    nGrid = "_GridSim"
end

nFileStr   = "KF_" * nNamePriorSpec * "_" * nDataSet * "_RClin1_GridNone"
nFileStrMH = "BSPF_" * nNamePriorSpec * "_" * nDataSet * "_RClin" * "$(nSolutionRCLinear)" * nGrid * "_Mhrun" *"$(mhrun)"
savedir      = "$(pwd())/PosteriorDraws/" * nFileStrMH * "/"
try mkdir(savedir) catch; end  # Create directory, if it exists skip creation


#--------------------------------------------------------------------------
# Construct proposal covariance matrix
#--------------------------------------------------------------------------

nLoadCovVar = 1;

if nLoadCovVar == 0
    readDir       = "$(pwd())/PriorDraws/"
    readName      = "SummaryDirectDrawsPrior1.csv"
    mSummaryDraws = CSV.read(readDir * readName ; datarow=1)
    mSummaryDraws = convert(Array,mSummaryDraws)
    vStdDevTheta  = mSummaryDraws[2,:]
    mSigmaProp    = Diagonal(vStdDevTheta.^2)
else
    readDir       = "$(pwd())/PosteriorDraws/" * nFileStr * "_Mhrun1/"
    readName      = nFileStr * "_Mhrun1_CovVarMatrix.csv"
    mSigmaProp    = CSV.read(readDir * readName ; datarow=1)
    mSigmaProp    = convert(Array,mSigmaProp)
end

#--------------------------------------------------------------------------
# Initial Value for MCMC
#--------------------------------------------------------------------------

readDir      = "$(pwd())/PosteriorDraws/"
readName     = nFileStrMH * "_InitialValue.csv"
vParameters  = CSV.read(readDir * readName , datarow=1 , limit=1)
vParameters  = convert(Array,vParameters)'


run_MCMC_BSPF(ME_scale, nM, nSimRWMH, mhrun, nBurnRWMH,
           silent, nOutput, c, nNamePriorSpec, mData, nStartDate, nEndDate,
           nFileStr, nFileStrMH, vParameters, mSigmaProp, rnd_off, nSolutionRCLinear,
           fixed_grid,vSmolyakGridSpec,options_int_use,options_float_use)
