#--------------------------------------------------------------------------
# Script to:
# Generate draws from prior by direct sampling
#--------------------------------------------------------------------------
using Atom
using Distributions
#using Clustering
#using Optim
#using LsqFit
#using Distances
using Random
using CSV
using DataFrames

clearconsole()

#--------------------------------------------------------------------------
# Select prior specification
#--------------------------------------------------------------------------
nNamePriorSpec = "Prior1"
include("PriorSpec/" * nNamePriorSpec * ".jl")
# prior specification is in mPriorSpec

#--------------------------------------------------------------------------
# Include relevant procedures
#--------------------------------------------------------------------------
include("FunctionsEstimation/fPriorDraws.jl")

#--------------------------------------------------------------------------
# Direct sampling from the prior
#--------------------------------------------------------------------------
nNumDraw = 10000;  # Select number of draws

nRNG = Random.seed!(123)
mPriorDraws = fPriorDraws(nNumDraw,mPriorSpec)

saveDir      = "$(pwd())/PriorDraws/"
saveName     = "DirectDraws" * nNamePriorSpec * ".csv"
CSV.write(saveDir * saveName,  DataFrame(mPriorDraws), writeheader=false)

mDraws        = convert(Array,mPriorDraws)

# Get CovVar Matrix from draws
mCovMat      = cov(mDraws)
saveName     = "CovVarMatrixDirectDraws" * nNamePriorSpec * ".csv"
CSV.write(saveDir * saveName,  DataFrame(mCovMat), writeheader=false)

#--------------------------------------------------------------------------
# Compute means and variances
#--------------------------------------------------------------------------
vMeanTheta   = mean(mDraws; dims = 1)
vStdDevTheta = std(mDraws; dims = 1)

#--------------------------------------------------------------------------
# HPD intervals: take as input posterior draws
#--------------------------------------------------------------------------
include("$(pwd())/FunctionsEstimation/" * "fHPDinterval.jl")
nHPDprob = 0.90;
mHPDinterval = fHPDinterval(mDraws,nHPDprob)

#--------------------------------------------------------------------------
# Save the results
#--------------------------------------------------------------------------
mOutput = [vMeanTheta; vStdDevTheta; mHPDinterval]
saveName     = "SummaryDirectDraws" * nNamePriorSpec * ".csv"
CSV.write(saveDir * saveName,  DataFrame(mOutput), writeheader=false)
